module Elabs
  module Concerns
    module CountableEntity
      extend ActiveSupport::Concern

      # List of lists like ['albums', 'uploads'] if 'uploads_counter' is in 'albums' table
      ADDITIONAL_HABTM_COUNTER_CACHES = [].freeze

      included do
        before_save :update_counter_caches?
        after_save :update_counter_caches

        before_destroy :update_counter_caches_on_delete?
        after_destroy :update_counter_caches
      end

      private

      def update_counter_caches?
        @action_on_counter_caches = if made_publicly_visible? && !persisted?
                                      :increment_but_tags
                                    elsif made_publicly_visible? && persisted?
                                      :increment
                                    elsif made_publicly_invisible?
                                      :decrement
                                    else
                                      :nothing
                                    end
      end

      def update_counter_caches_on_delete?
        @action_on_counter_caches = if publicly_visible?
                                      :decrement
                                    else
                                      :nothing
                                    end
      end

      def update_counter_caches
        if @action_on_counter_caches == :increment
          increment_counter_caches
        elsif @action_on_counter_caches == :increment_but_tags
          increment_counter_caches false
        elsif @action_on_counter_caches == :decrement
          decrement_counter_caches
        end
      end

      # rubocop:disable Metrics/AbcSize
      def increment_counter_caches(update_created_on_the_fly = true)
        User.increment_counter "#{self.class.table_name}_count", user.id
        Language.increment_counter "#{self.class.table_name}_count", language.id
        License.increment_counter "#{self.class.table_name}_count", license.id

        return unless update_created_on_the_fly

        tags.each { |tag| tag.increment! "#{self.class.table_name}_count" }

        self.class::ADDITIONAL_HABTM_COUNTER_CACHES.each do |relation|
          send(relation[0]).each { |entity| entity.increment! "#{relation[1]}_count" }
        end
      end

      # rubocop:enable Metrics/AbcSize

      # rubocop:disable Metrics/AbcSize
      def decrement_counter_caches
        User.decrement_counter "#{self.class.table_name}_count", user.id
        Language.decrement_counter "#{self.class.table_name}_count", language.id
        License.decrement_counter "#{self.class.table_name}_count", license.id

        tags.each { |tag| tag.decrement! "#{self.class.table_name}_count" }

        self.class::ADDITIONAL_HABTM_COUNTER_CACHES.each do |relation|
          send(relation[0]).each { |entity| entity.decrement! "#{relation[1]}_count" }
        end
      end
      # rubocop:enable Metrics/AbcSize
    end
  end
end
